#include "JoyDevice.h"
#include "windows.h"
#include <stdio.h>

/*****************************************************************
 * JoyDevice Set-Up functions                                    *
 *****************************************************************/
BOOL chkPosState(jint jid, jint *amin, jint *amax, jdouble *axis, jlong *b) {
   JOYINFOEX jpos;
   ZeroMemory(&jpos, sizeof(JOYINFOEX));
   jpos.dwSize  = sizeof(JOYINFOEX);
   jpos.dwFlags = JOY_RETURNALL;

   if (joyGetPosEx(jid, &jpos) == JOYERR_NOERROR) {

		axis[0]  = (jdouble) jpos.dwXpos - amin[0];
      axis[0] /= (jdouble) amax[0]     - amin[0];
      axis[0] *= (jdouble) 100.00;

		axis[1]  = (jdouble) jpos.dwYpos - amin[1];
      axis[1] /= (jdouble) amax[1]     - amin[1];
      axis[1] *= (jdouble) 100.00;

      if ((amax[2] - amin[2]) > 0) {
         axis[2]  = (jdouble) jpos.dwZpos - amin[2];
         axis[2] /= (jdouble) amax[2]     - amin[2];
         axis[2] *= (jdouble) 100.00;
         
      } else {
         axis[2] = (jdouble) 0.00;
      }

      *b = (jlong)jpos.dwButtons;

      return TRUE;
   }

   return FALSE;
}


JNIEXPORT jboolean JNICALL Java_JoyDevice_updateJoy(
   JNIEnv *env,
   jobject obj)
{

   jclass       cls;
   jfieldID     fID;
   jintArray    ia;
   jdoubleArray da;
   jint         jid;
   jlong        btns;
   jint         aminBuf[3];
   jint         amaxBuf[3];
   jdouble      axisBuf[3];


   //get class ID
   cls = (*env)->GetObjectClass(env, obj);

   //get min array
   fID = (*env)->GetFieldID(env, cls, "amin", "[I");
   ia  = (jintArray) (*env)->GetObjectField(env, obj, fID);
   (*env)->GetIntArrayRegion(env, ia, (jsize)0, (jsize) 3, aminBuf);

   //get max array
   fID = (*env)->GetFieldID(env, cls, "amax", "[I");
   ia  = (jintArray) (*env)->GetObjectField(env, obj, fID);
   (*env)->GetIntArrayRegion(env, ia, (jsize)0, (jsize) 3, amaxBuf);

   //get joyID array
   fID = (*env)->GetFieldID(env, cls, "joyDeviceID", "I");
   jid = (*env)->GetIntField(env, obj, fID);


   //Get JOY POS INFO
   if (chkPosState(jid, aminBuf, amaxBuf, axisBuf, &btns) == FALSE)
      return JNI_FALSE;


   //Update Axis Array
   fID = (*env)->GetFieldID(env, cls, "axis", "[D");
   da  = (jdoubleArray) (*env)->GetObjectField(env, obj, fID);
   (*env)->SetDoubleArrayRegion(env, da, (jsize)0, (jsize) 3, axisBuf);

   //update buttons
   fID = (*env)->GetFieldID(env, cls, "buttons", "J");
   (*env)->SetLongField(env, obj, fID, btns);

   return JNI_TRUE;
}




/*****************************************************************
 * JoyDevice Set-Up functions                                    *
 *****************************************************************/
BOOL chkCapState(jint jid, jint *amin, jint *amax) {
   JOYCAPS jinf;

   if (joyGetDevCaps(jid, &jinf, sizeof(JOYCAPS)) == JOYERR_NOERROR) {
      amin[0] = (jint)jinf.wXmin;
      amax[0] = (jint)jinf.wXmax;

      amin[1] = (jint)jinf.wYmin;
      amax[1] = (jint)jinf.wYmax;

      amin[2] = (jint)jinf.wZmin;
      amax[2] = (jint)jinf.wZmax;

      return TRUE;
   }

   return FALSE;
}


JNIEXPORT jboolean JNICALL Java_JoyDevice_prepJoy(
   JNIEnv    *env,
   jobject    obj,
   jint       joyid)
{
   jclass     cls;
   jfieldID   aminID;
   jfieldID   amaxID;
   jintArray  aminArray;
   jintArray  amaxArray;
   jint       aminBuf[3];
   jint       amaxBuf[3];


   //get min/max arrays
   cls    = (*env)->GetObjectClass(env, obj);
   aminID = (*env)->GetFieldID(env, cls, "amin", "[I");
   amaxID = (*env)->GetFieldID(env, cls, "amax", "[I");

   aminArray = (jintArray) (*env)->GetObjectField(env, obj, aminID);
   amaxArray = (jintArray) (*env)->GetObjectField(env, obj, amaxID);


   //Get JOY CAPS INFO
   if (chkCapState(joyid, aminBuf, amaxBuf) == FALSE)
      return JNI_FALSE;

   (*env)->SetIntArrayRegion(env, aminArray, (jsize)0, (jsize) 3, aminBuf);
   (*env)->SetIntArrayRegion(env, amaxArray, (jsize)0, (jsize) 3, amaxBuf);

   return JNI_TRUE;
}


